@@ -1,5 +1,5 @@
 import requests
-from requests.utils import dict_from_cookiejar
+from requests.utils import dict_from_cookiejar, cookiejar_from_dict
 
 from hmac import HMAC
 from hashlib import sha1
@@ -12,8 +12,7 @@ from .jstime import epoch
 import logging
 logger = logging.getLogger(__name__)
 
-from urllib import quote
-from urllib2 import urlparse
+from urllib2 import urlparse, quote, unquote
 
 
 def js_compatible_quote(string):
@@ -24,10 +23,20 @@ def js_compatible_quote(string):
     return quote(string, safe='~@#$&()*!+=:;,.?/\'')
 
 
+def get_json(resp):
+    """
+    Deal with a breaking change in requests 1.0. We don't know if we will need
+    to ask for resp.json or resp.json() unless we do this.
+    """
+    if callable(resp.json):
+        return resp.json()
+    else:
+        return resp.json
+
 class LuminosoAuth(object):
     """Wraps REST requests with Luminoso's required authentication parameters"""
     def __init__(self, username, password, url=URL_BASE,
-                 validity_ms=30000, auto_login=False, proxies=None):
+                 validity_ms=120000, auto_login=False, proxies=None):
         """Log-in to the Luminoso API
 
            username, password => (str) credentials to access the server
@@ -48,11 +57,23 @@ class LuminosoAuth(object):
         self._validity_ms = validity_ms
 
         # Initialize the requests session
-        self._session = requests.session(proxies=proxies)
+        self._session = requests.session()
+        if proxies is not None:
+            self._session.proxies = proxies
 
         # Fetch session credentials
         self.login(username, password)
 
+    def no_retry_copy(self):
+        """
+        Return a duplicate LuminosoAuth object that will not retry a connection
+        """
+        return LuminosoAuth(self.username, self.password,
+                            url=self.url,
+                            validity_ms=self._validity_ms,
+                            auto_login=False,
+                            proxies=self._session.proxies)
+
     def login(self, username, password):
         """Fetch a session key to use in this authentication context"""
         params = {'username': username, 'password': password}
@@ -61,24 +82,45 @@ class LuminosoAuth(object):
         # Make sure the session is valid
         if resp.status_code == 401:
             logger.error('%s gave response %r' % (resp.url, resp.text))
-            raise LuminosoLoginError
+            raise LuminosoLoginError(resp.text)
 
         # Save the session cookie
         self._session_cookie = resp.cookies['session']
-        logger.info('Cookie: %r', self._session_cookie)
 
         # Save the key_id
-        self._key_id = resp.json['result']['key_id']
-        self._secret = resp.json['result']['secret']
+        self._key_id = get_json(resp)['result']['key_id']
+        self._secret = get_json(resp)['result']['secret']
 
     def __on_response(self, resp):
         """Handle auto-login and update session cookies"""
         if resp.status_code == 401:
             if self._auto_login:
-                raise NotImplementedError
+                logger.info('request failed with 401; retrying with fresh login')
+                # Do not enter an infinite retry loop
+                retry_auth = self.no_retry_copy()
+                resp.request.deregister_hook('response', self.__on_response)
+
+                # Re-issue the request
+                resp.request.auth = retry_auth
+                resp.request.send(anyway=True)
+
+                # Save the new credentials if successful
+                new_result = resp.request.response.status_code
+                if 200 <= new_result < 300:
+                    # Save the new credentials
+                    logger.info('retry successful')
+                    self._key_id = retry_auth._key_id
+                    self._secret = retry_auth._secret
+                    self._session_cookie = retry_auth._session_cookie
+
+                    # Return the new result
+                    return resp.request.response
+                else:
+                    logger.error('retry failed')
+                    return resp
 
         self._session_cookie = dict_from_cookiejar(resp.cookies)['session']
-        logger.info('Cookie: %r', self._session_cookie)
+        logger.debug('Cookie: %r', self._session_cookie)
 
         return resp
 
@@ -95,6 +137,7 @@ class LuminosoAuth(object):
         pathstring = req.path_url.split('?')[0]
         if not pathstring.endswith('/'):
             pathstring += '/'
+        pathstring = unquote(pathstring)
         # Build the list
         signing_list = [req.method,
                         self._host,
@@ -115,6 +158,7 @@ class LuminosoAuth(object):
     def __call__(self, req):
         # Register the on_response hook
         req.register_hook('response', self.__on_response)
+        logger.debug('auto_login is %s', 'on' if self._auto_login else 'off')
 
         # Determine the expiry
         expiry = epoch() + self._validity_ms
@@ -122,6 +166,11 @@ class LuminosoAuth(object):
         # Set the key id
         req.params['key_id'] = self._key_id
 
+        # Remove auth fields
+        for field in ('expires', 'sig'):
+            if field in req.params:
+                req.params.pop(field)
+
         # Determine if this is an upload
         if isinstance(req.data, dict):
             params = req.params.copy()
@@ -146,6 +195,9 @@ class LuminosoAuth(object):
         req.params['sig'] = sig
 
         # Load the session cookie into the request
-        req.cookies['session'] = self._session_cookie
+        req.cookies = cookiejar_from_dict({'session': self._session_cookie})
+        if 'Cookie' in req.headers:
+            req.headers.pop('Cookie')
+            req.headers._lower_keys = None
 
         return req
