@@ -55,7 +55,10 @@ class LuminosoClient(object):
         the authentication for you.
         """
         self._auth = auth
-        self._session = requests.session(auth=auth, proxies=proxies)
+        self._session = requests.session()
+        self._session.auth = auth
+        if proxies is not None:
+            self._session.proxies = proxies
         self.url = ensure_trailing_slash(url)
         self.root_url = root_url or get_root_url(url)
 
@@ -64,7 +67,7 @@ class LuminosoClient(object):
 
     @staticmethod
     def connect(url='/', username=None, password=None, root_url=None,
-                proxies=None):
+                proxies=None, auto_login=True):
         """
         Returns an object that makes requests to the API, authenticated
         with the provided username/password, at URLs beginning with `url`.
@@ -75,6 +78,11 @@ class LuminosoClient(object):
 
         `proxies` is a dictionary from URL schemes (like 'http') to proxy
         servers, in the same form used by the `requests` module.
+
+        By default, this object will automatically re-login if its
+        authentication times out, which happens after ten minutes of inactivity
+        or one hour, whichever comes first. If for security reasons you do not
+        want this to happen, set `auto_login` to False.
         """
         if url.startswith('/'):
             url = URL_BASE + url
@@ -88,7 +96,8 @@ class LuminosoClient(object):
             password = getpass('Password for %s: ' % username)
 
         logger.info('creating LuminosoAuth object')
-        auth = LuminosoAuth(username, password, url=root_url, proxies=proxies)
+        auth = LuminosoAuth(username, password, url=root_url, proxies=proxies,
+                            auto_login=auto_login)
         return LuminosoClient(auth, url)
 
     def _request(self, req_type, url, **kwargs):
@@ -105,7 +114,7 @@ class LuminosoClient(object):
             error = result.text
             if result.status_code == 401:
                 error_class = LuminosoAuthError
-            if result.status_code in (404, 405):
+            elif result.status_code in (404, 405):
                 error_class = LuminosoClientError
             elif result.status_code >= 500:
                 error_class = LuminosoServerError
@@ -324,14 +333,14 @@ class LuminosoClient(object):
         newclient = LuminosoClient(self._auth, self.root_url, self.root_url)
         return newclient.post('ping')
 
-    def upload(self, path, docs):
+    def upload(self, path, docs, **params):
         """
         A convenience method for uploading a set of dictionaries representing
         documents. You still need to specify the URL to upload to, which will
         look like ROOT_URL/myname/projects/projectname/docs.
         """
-        json_data = json.dumps(docs)
-        return self.post_data(path, json_data, 'application/json')
+        json_data = json.dumps(list(docs))
+        return self.post_data(path, json_data, 'application/json', **params)
 
     def wait_for(self, job_id, base_path=None, interval=5):
         """
@@ -364,7 +373,6 @@ class LuminosoClient(object):
             if count % 10 == 0:
                 self.keepalive()
             response = self.get(path)
-            logger.info(response)
             if response['stop_time']:
                 return response
             time.sleep(interval)
@@ -407,4 +415,3 @@ def jsonify_parameters(params):
         else:
             result[param] = json.dumps(value)
     return result
-
